@@ -108,6 +108,12 @@ class GroupMap:
     def num_elements(self):
         return self._num_elements
 
+    def to_dict(self):
+        return {
+            group_name: list(self[group_name].nonzero()[0])
+            for group_name in self.keys()
+        }
+
     def mask_for_element(self, element):
         """
         Get the read-only group mask for the requested element.
@@ -181,3 +187,49 @@ class GroupMap:
             masks=np.asarray(self._masks[:, f_new_to_old]),
             copy_masks=False,
         )
+
+    def defragment(self, group_order=None):
+        """
+        Reorder the faces to keep groups together.
+
+        Overlapping groups are not supported.
+
+        Args:
+            group_order (list): The desired order of the groups. The default is
+                to order by the first face in which the group appears.
+
+        Returns:
+            np.ndarray: The new order of the faces, suitable for passing to
+            `lacecore.reindex_faces()`.
+        """
+        from collections import Counter
+
+        if group_order is None:
+            # Inspired by https://stackoverflow.com/a/22150003/893113
+            group_order = [item[0] for item in Counter(self._group_names).most_common()]
+            group_order.reverse()
+        else:
+            nonempty_groups = [
+                group_name for group_name in self if np.any(self[group_name])
+            ]
+            missing_groups = set(nonempty_groups) - set(group_order)
+            if len(missing_groups) > 0:
+                raise ValueError(
+                    f"group_order is missing groups: {', '.join(sorted(list(missing_groups)))}"
+                )
+            unknown_groups = set(group_order) - set(self.keys())
+            if len(unknown_groups) > 0:
+                raise ValueError(
+                    f"group_order contains unknown groups: {', '.join(sorted(list(unknown_groups)))}"
+                )
+
+        ordering = np.repeat(-1, self.num_elements)
+        next_index = 0
+        for group_name in group_order:
+            this_mask = self[group_name]
+            num_these_elements = np.count_nonzero(this_mask)
+            if not np.all(ordering[this_mask] == -1):
+                raise ValueError(f'Group "{group_name}" overlaps with previous groups')
+            ordering[this_mask] = np.arange(next_index, next_index + num_these_elements)
+            next_index += num_these_elements
+        return ordering
